/* FILE: exchangeptwise.h            -*-Mode: c++-*-
 *
 * 6 neighbor exchange energy on rectangular mesh, with
 * exchange coefficient A specified on a cell-by-cell
 * basis.  The effective coefficient between two neighboring
 * cells is calculated on the fly via 2.A_1.A_2/(A_1+A_2)
 */

#ifndef _OXS_EXCHANGEPTWISE
#define _OXS_EXCHANGEPTWISE

#include "atlas.h"
#include "key.h"
#include "energy.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "rectangularmesh.h"
#include "scalarfield.h"
#include "util.h"

/* End includes */

class Oxs_ExchangePtwise:public Oxs_Energy {
private:
  Oxs_OwnedPointer<Oxs_ScalarField> A_init;
  mutable UINT4m mesh_id;
  mutable Oxs_MeshValue<REAL8m> A;
  /// Exchange coefficient A is filled by A_init routine
  /// each time a change in the mesh is detected.

protected:
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const;

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_ExchangePtwise(const char* name,     // Child instance id
		    Oxs_Director* newdtr, // App director
		    const char* argstr);  // MIF input block parameters
  virtual ~Oxs_ExchangePtwise();
};


#endif // _OXS_EXCHANGEPTWISE
