/* FILE: randomscalarfield.cc      -*-Mode: c++-*-
 *
 * Random scalar field object, derived from Oxs_ScalarField class.
 *
 */

#include "oc.h"
#include "nb.h"

#include "threevector.h"
#include "meshvalue.h"
#include "director.h"
#include "randomscalarfield.h"
#include "energy.h"		// Needed to make MSVC++ 5 happy

// Oxs_Ext registration support
OXS_EXT_REGISTER(Oxs_RandomScalarField);

/* End includes */


// Constructor
Oxs_RandomScalarField::Oxs_RandomScalarField(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  const char* argstr)   // MIF input block parameters
  : Oxs_ScalarField(name,newdtr,argstr)
{
  // Process arguments.
  range_min=GetRealInitValue("range_min");
  range_max=GetRealInitValue("range_max");

  VerifyAllInitArgsUsed();

  if(range_min>range_max) {
    char buf[1024];
    Oc_Snprintf(buf,sizeof(buf),
		"Requested range_min value (%g) bigger than range_max (%g)",
		range_min,range_max);
    throw Oxs_Ext::Error(this,buf);
  }
}

BOOL Oxs_RandomScalarField::Init()
{
  return 1;
}

REAL8m Oxs_RandomScalarField::Value(const ThreeVector& /* pt */) const
{
  if(range_min>=range_max) return range_max; // No spread
  return (range_max-range_min)*Oc_UnifRand() + range_min;
}

void
Oxs_RandomScalarField::FillMeshValue
(const Oxs_Mesh* mesh,
 Oxs_MeshValue<REAL8m>& array) const
{
  array.AdjustSize(mesh);
  UINT4m i,size=mesh->Size();
  if(range_min>=range_max) {
    // No spread
    for(i=0;i<size;i++) {
      array[i] = range_max;
    }
  } else {
    REAL8m spread = range_max - range_min;
    for(i=0;i<size;i++) {
      array[i] = spread*Oc_UnifRand() + range_min;
    }
  }
}
