/* FILE: scriptatlas.h                 -*-Mode: c++-*-
 *
 * Script atlas class, derived from Oxs_Atlas class.
 *
 */

#ifndef _OXS_SCRIPTATLAS
#define _OXS_SCRIPTATLAS

#include <string>
#include <vector>

#include "atlas.h"
#include "ext.h"
#include "tclcommand.h"
#include "threevector.h"
#include "util.h"
/* End includes */

class Oxs_ScriptAtlas:public Oxs_Atlas {
private:
  vector<string> region_name_list;
  Oxs_Box bounding_box;
  Oxs_ThreeVector scale;
  vector<Oxs_TclCommandLineOption> command_options;
  Oxs_TclCommand cmd;
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init() { return 1; }

  Oxs_ScriptAtlas(const char* name,
		  Oxs_Director* newdtr,
		  const char* argstr);   // MIF block argument string

  ~Oxs_ScriptAtlas();

  void GetWorldExtents(Oxs_Box &mybox) const {
    mybox = bounding_box;
  }

  BOOL GetRegionExtents(UINT4m id,Oxs_Box &mybox) const;
  /// Fills mybox with a bounding box for region with specified
  /// id number.  Return value is 0 iff id is invalid.  At present,
  /// given a valid id number, just returns the world bounding
  /// box.  In the future, we might want to add support for refined
  /// region bounding boxes.

  INT4m GetRegionId(const ThreeVector& point) const;
  /// Returns the id number for the region in
  /// containing the point, as determined by the
  /// script.  The return value is 0 if the
  /// point is not contained in the atlas, i.e,
  /// belongs to the "universe".

  INT4m GetRegionId(const string& name) const;
  /// Given a region id string (name), returns
  /// the corresponding region id index.  If
  /// "name" is not included in the atlas, then
  /// -1 is returned.  Note: If name == "universe",
  /// then the return value will be 0.

  BOOL GetRegionName(UINT4m id,string& name) const;
  /// Given an id number, fills in "name" with
  /// the corresponding region id string.  Returns
  /// 1 on success, 0 if id is invalid.  If id is 0,
  /// then name is set to "universe", and the return
  /// value is 1.

  UINT4m GetRegionCount() const {
    return UINT4m(region_name_list.size());
  }
  /// Valid RegionId numbers range from 0 to GetRegionCount() - 1,
  /// inclusive, where 0 is the special "universe" region.

};

#endif // _OXS_SCRIPTATLAS
