/* FILE: uniformexchange.h            -*-Mode: c++-*-
 *
 * Uniform 6 neighbor exchange energy on rectangular mesh,
 * derived from Oxs_Energy class.
 *
 */

#ifndef _OXS_UNIFORMEXCHANGE
#define _OXS_UNIFORMEXCHANGE

#include "oc.h"
#include "director.h"
#include "energy.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"

/* End includes */

class Oxs_UniformExchange:public Oxs_Energy {
private:
  REAL8m A;

  enum ExchangeKernel { UNKNOWN, NGBR_6, NGBR_12,
			NGBR_12_MIRROR, NGBR_26 } kernel;
  /// Exchange formulation to use.  "unknown" is invalid; it
  /// is defined for error detection.

  // Calculation routines for each of the
  // aforementioned energy formulations.
  void CalcEnergy6Ngbr
  (const Oxs_MeshValue<ThreeVector>& spin,
   const Oxs_MeshValue<REAL8m>& Ms_inverse,
   const Oxs_RectangularMesh* mesh,
   Oxs_MeshValue<REAL8m>& energy,
   Oxs_MeshValue<ThreeVector>& field) const;
  void CalcEnergy12Ngbr
  (const Oxs_MeshValue<ThreeVector>& spin,
   const Oxs_MeshValue<REAL8m>& Ms_inverse,
   const Oxs_RectangularMesh* mesh,
   Oxs_MeshValue<REAL8m>& energy,
   Oxs_MeshValue<ThreeVector>& field) const;
  void CalcEnergy12NgbrMirror
  (const Oxs_MeshValue<ThreeVector>& spin,
   const Oxs_MeshValue<REAL8m>& Ms_inverse,
   const Oxs_RectangularMesh* mesh,
   Oxs_MeshValue<REAL8m>& energy,
   Oxs_MeshValue<ThreeVector>& field) const;
  void CalcEnergy26Ngbr
  (const Oxs_MeshValue<ThreeVector>& spin,
   const Oxs_MeshValue<REAL8m>& Ms_inverse,
   const Oxs_RectangularMesh* mesh,
   Oxs_MeshValue<REAL8m>& energy,
   Oxs_MeshValue<ThreeVector>& field) const;


protected:
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const;

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_UniformExchange(const char* name,     // Child instance id
		    Oxs_Director* newdtr, // App director
		    const char* argstr);  // MIF input block parameters
  virtual ~Oxs_UniformExchange();
  virtual BOOL Init();
};


#endif // _OXS_UNIFORMEXCHANGE
